@@ -7,6 +7,8 @@ import re
 import singer
 import stat
 import time
+import gzip
+import zipfile
 from datetime import datetime
 from paramiko.ssh_exception import AuthenticationException, SSHException
 
@@ -107,10 +109,17 @@ class SFTPConnection():
             else:
                 if is_empty(file_attr):
                     continue
+
+                last_modified = file_attr.st_mtime
+                if last_modified is None:
+                    LOGGER.warning("Cannot read m_time for file %s, defaulting to current epoch time",
+                                   os.path.join(prefix, file_attr.filename))
+                    last_modified = datetime.utcnow().timestamp()
+
                 # NB: SFTP specifies path characters to be '/'
                 #     https://tools.ietf.org/html/draft-ietf-secsh-filexfer-13#section-6
                 files.append({"filepath": prefix + '/' + file_attr.filename,
-                              "last_modified": datetime.utcfromtimestamp(file_attr.st_mtime).replace(tzinfo=pytz.UTC)})
+                              "last_modified": datetime.utcfromtimestamp(last_modified).replace(tzinfo=pytz.UTC)})
 
         return files
 
@@ -134,11 +143,22 @@ class SFTPConnection():
         if modified_since is not None:
             matching_files = [f for f in matching_files if f["last_modified"] > modified_since]
 
-        return matching_files
+        # sort files in increasing order of "last_modified"
+        sorted_files = sorted(matching_files, key = lambda x: (x['last_modified']).timestamp())
+        return sorted_files
 
     def get_file_handle(self, f):
-        """ Takes a file dict {"filepath": "...", "last_modified": "..."} and returns a handle to the file. """
-        return self.sftp.open(f["filepath"], 'rb')
+        """ Takes a file dict {"filepath": "...", "last_modified": "..."}
+        -> returns a handle to the file.
+        -> raises error with appropriate logger message """
+        try:
+            return self.sftp.open(f["filepath"], 'rb')
+        except OSError as e:
+            if "Permission denied" in str(e):
+                LOGGER.warn("Skipping %s file because you do not have enough permissions.", f["filepath"])
+            else:
+                LOGGER.warn("Skipping %s file because it is unable to be read.", f["filepath"])
+            raise
 
     def get_files_matching_pattern(self, files, pattern):
         """ Takes a file dict {"filepath": "...", "last_modified": "..."} and a regex pattern string, and returns files matching that pattern. """
